__version__ = "0.1.2"
__author__ = "Forest Dussault"
__email__ = "forest.dussault@canada.ca"

DEPENDENCIES = [
    'skesa',
    'bbduk.sh',
    'bbmap.sh',
    'tadpole.sh',
    'pilon',
    'samtools'
]
